@@ -15,7 +15,7 @@ use vars qw($VERSION @ISA $makeMissing);
 use Image::ExifTool qw(:Utils :Vars);
 use Image::ExifTool::XMP;
 
-$VERSION = '1.31';
+$VERSION = '1.33';
 @ISA = qw(Exporter);
 
 # set this to a language code to generate Lang module with 'MISSING' entries
@@ -58,6 +58,12 @@ my %translateLang = (
 my $numbersFirst = 1;   # set to -1 to sort numbers last, or 2 to put negative numbers last
 my $caseInsensitive;    # used internally by sort routine
 
+# write groups that don't represent real family 1 group names
+my %fakeWriteGroup = (
+    Comment => 1,   # (JPEG Comment)
+    colr => 1,      # (Jpeg2000 'colr' box)
+);
+
 #------------------------------------------------------------------------------
 # Utility to print tag information database as an XML list
 # Inputs: 0) output file name (undef to send to console),
@@ -164,6 +170,7 @@ PTILoop:    for ($index=0; $index<@infoArray; ++$index) {
                 $format = 'struct' if $$tagInfo{Struct};
                 if (defined $format) {
                     $format =~ s/\[.*\$.*\]//;   # remove expressions from format
+                    $format = 'undef' if $format eq '2'; # (special case)
                 } elsif ($isBinary) {
                     $format = 'int8u';
                 } else {
@@ -177,9 +184,8 @@ PTILoop:    for ($index=0; $index<@infoArray; ++$index) {
                 }
                 my @groups = $et->GetGroup($tagInfo);
                 my $writeGroup = $$tagInfo{WriteGroup} || $$table{WRITE_GROUP};
-                if ($writeGroup and $writeGroup ne 'Comment') {
-                    $groups[1] = $writeGroup;   # use common write group for group 1
-                }
+                # use common write group for group 1 (unless fake)
+                $groups[1] = $writeGroup if $writeGroup and not $fakeWriteGroup{$writeGroup};
                 # add group names if different from table defaults
                 my $grp = '';
                 for ($fam=0; $fam<3; ++$fam) {
@@ -198,6 +204,8 @@ PTILoop:    for ($index=0; $index<@infoArray; ++$index) {
                     push @flags, 'Permanent' if $$tagInfo{Permanent} or
                         ($groups[0] eq 'MakerNotes' and not defined $$tagInfo{Permanent});
                     $grp = " flags='" . join(',', sort @flags) . "'$grp" if @flags;
+                    # add parent structure tag ID
+                    $grp .= " struct='$$tagInfo{ParentTagInfo}{TagID}'" if $$tagInfo{ParentTagInfo};
                 }
                 print $fp " <tag id='${xmlID}' name='${name}'$ind type='${format}'$count writable='${writable}'$grp";
                 if ($opts{NoDesc}) {
@@ -637,7 +645,7 @@ and values.
 
 ~head1 AUTHOR
 
-Copyright 2003-2021, Phil Harvey (philharvey66 at gmail.com)
+Copyright 2003-2022, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -821,7 +829,7 @@ Number of modules updated, or negative on error.
 
 =head1 AUTHOR
 
-Copyright 2003-2021, Phil Harvey (philharvey66 at gmail.com)
+Copyright 2003-2022, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
